open Format
open Lexing

let syntax_error p =
  eprintf "File %S at line %d, character %d:@.Syntax error.@." 
    p.pos_fname p.pos_lnum (p.pos_cnum - p.pos_bol)

let finally x f g =
  match f x with
  | y           -> g x; y
  | exception e -> g x; raise e

let open_in_do ?path f =
  match path with 
  | None -> f stdin
  | Some file -> finally (open_in file) f close_in

let read_file parse lex filename =
  let f ch = 
    let lexbuf = from_channel ch in
    let lex_curr_p = 
      { lexbuf.lex_curr_p with pos_fname = filename } in
    try
      parse lex { lexbuf with lex_curr_p = lex_curr_p }
    with Parsing.Parse_error -> 
      (syntax_error lexbuf.lex_curr_p; exit 1)
  in
  open_in_do ~path:filename f


let read_trs filename = 
  read_file Parser.toplevel Lexer.token filename
